clear, clc;

% This is an example for running the function nnLeastR
% 
%  min  1/2 || A x - y||^2 + 1/2 * rsL2 * ||x||_2^2 + rho * ||x||_1 
%  s.t. x>=0
%
% For detailed description of the function, please refer to the Manual.
%
% Last modified on August 10, 2009.
%
% For any problem, please contact Jun Liu (j.liu@asu.edu)

cd ..                % changes the current working 
                     % directory to the directory above it
root=cd;             
addpath(genpath([root '/SLEP']));
                     % add the functions in the folder SLEP to the path
                     
cd Examples          % set the working directory back

m=1000;  n=1000;    % The data matrix is of size m x n

% for reproducibility
randNum=1;

% ---------------------- Generate random data ----------------------
randn('state',(randNum-1)*3+1);
A=randn(m,n);       % the data matrix

randn('state',(randNum-1)*3+2);
xOrin=randn(n,1);

randn('state',(randNum-1)*3+3);
noise=randn(m,1);
y=A*xOrin +...
    noise*0.01;     % the response

rho=0.1;            % the regularization parameter
                    % it is a ratio between (0,1), if .rFlag=1

%----------------------- Set optional items ------------------------
opts=[];

% Starting point
opts.init=2;        % starting from a zero point

% termination criterion
opts.tFlag=5;       % run .maxIter iterations
opts.maxIter=100;   % maximum number of iterations

% normalization
opts.nFlag=0;       % without normalization

% regularization
opts.rFlag=1;       % the input parameter 'rho' is a ratio in (0, 1)
opts.rsL2=0;        % the squared two norm term

%----------------------- Run the code LeastR -----------------------
[x, funVal]= nnLeastR(A, y, rho, opts);

figure;
plot(funVal);
xlabel('Iteration (i)');
ylabel('The objective function value');

% --------------------- compute the pathwise solutions ----------------
opts.fName='nnLeastR';      % set the function name to 'LeastR'
Z=[0.5, 0.2, 0.1, 0.01];    % set the parameters

% run the function pathSolutionLeast
fprintf('\n Compute the pathwise solutions, please wait...');
X=pathSolutionLeast(A, y, Z, opts);